package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.StreamCipher;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithIV;

public class HC128Engine implements StreamCipher {
   private int[] p = new int[512];
   private int[] q = new int[512];
   private int cnt = 0;
   private byte[] key;
   private byte[] iv;
   private boolean initialised;
   private byte[] buf = new byte[4];
   private int idx = 0;

   private static int f1(int x) {
      return rotateRight(x, 7) ^ rotateRight(x, 18) ^ x >>> 3;
   }

   private static int f2(int x) {
      return rotateRight(x, 17) ^ rotateRight(x, 19) ^ x >>> 10;
   }

   private int g1(int x, int y, int z) {
      return (rotateRight(x, 10) ^ rotateRight(z, 23)) + rotateRight(y, 8);
   }

   private int g2(int x, int y, int z) {
      return (rotateLeft(x, 10) ^ rotateLeft(z, 23)) + rotateLeft(y, 8);
   }

   private static int rotateLeft(int x, int bits) {
      return x << bits | x >>> -bits;
   }

   private static int rotateRight(int x, int bits) {
      return x >>> bits | x << -bits;
   }

   private int h1(int x) {
      int[] var2;
      return (var2 = this.q)[x & 255] + var2[(x >> 16 & 255) + 256];
   }

   private int h2(int x) {
      int[] var2;
      return (var2 = this.p)[x & 255] + var2[(x >> 16 & 255) + 256];
   }

   private static int mod1024(int x) {
      return x & 1023;
   }

   private static int mod512(int x) {
      return x & 511;
   }

   private static int dim(int x, int y) {
      return mod512(x - y);
   }

   private int step() {
      int var1 = mod512(this.cnt);
      int var10000;
      int[] var10002;
      int[] var2;
      int var3;
      int var10004;
      int var10006;
      if (this.cnt < 512) {
         var10002 = var2 = this.p;
         var10004 = var2[var1];
         var10006 = var2[dim(var1, 3)];
         var3 = this.p[dim(var1, 10)];
         var10002[var1] = var10004 + this.g1(var10006, var3, this.p[dim(var1, 511)]);
         var10000 = this.h1(this.p[dim(var1, 12)]) ^ this.p[var1];
      } else {
         var10002 = var2 = this.q;
         var10004 = var2[var1];
         var10006 = var2[dim(var1, 3)];
         var3 = this.q[dim(var1, 10)];
         var10002[var1] = var10004 + this.g2(var10006, var3, this.q[dim(var1, 511)]);
         var10000 = this.h2(this.q[dim(var1, 12)]) ^ this.q[var1];
      }

      this.cnt = mod1024(this.cnt + 1);
      return var10000;
   }

   private void init() {
      // $FF: Couldn't be decompiled
   }

   private byte getByte() {
      int var1;
      if (this.idx == 0) {
         var1 = this.step();
         byte[] var2;
         byte[] var10000 = var2 = this.buf;
         var2[3] = (byte)(var1 & 255);
         int var10001 = var1 >> 8;
         var2[2] = (byte)(var1 >> 8 & 255);
         var10001 >>= 8;
         var2[1] = (byte)(var10001 & 255);
         var10000[0] = (byte)(var10001 >> 8 & 255);
      }

      byte var3 = this.buf[var1 = this.idx];
      this.idx = var1 + 1 & 3;
      return var3;
   }

   public String getAlgorithmName() {
      return "HC-128";
   }

   public void init(boolean forEncryption, CipherParameters params) {
      CipherParameters forEncryption1;
      if (params instanceof ParametersWithIV) {
         ParametersWithIV var10000 = (ParametersWithIV)params;
         this.iv = ((ParametersWithIV)params).getIV();
         forEncryption1 = var10000.getParameters();
      } else {
         this.iv = new byte[0];
         forEncryption1 = params;
      }

      if (forEncryption1 instanceof KeyParameter) {
         this.key = ((KeyParameter)forEncryption1).getKey();
         this.init();
         this.initialised = true;
      } else {
         throw new IllegalArgumentException("Invalid parameter passed to HC128 init - " + params.getClass().getName());
      }
   }

   public void processBytes(byte[] in, int inOff, int len, byte[] out, int outOff) {
      if (!this.initialised) {
         throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
      } else if (inOff + len > in.length) {
         throw new DataLengthException("input buffer too short");
      } else if (outOff + len > out.length) {
         throw new DataLengthException("output buffer too short");
      } else {
         for(int var6 = 0; var6 < len; ++var6) {
            int var7 = outOff + var6;
            out[var7] = (byte)(in[inOff + var6] ^ this.getByte());
         }

      }
   }

   public void reset() {
      this.idx = 0;
      this.init();
   }

   public byte returnByte(byte in) {
      return (byte)(in ^ this.getByte());
   }
}
